import { ul, li, input } from "@hyperapp/html"

const withEnterKey = (action) => (state, payload) => {
  if (payload.key && payload.key === "Enter") return [action, payload]
  return state
}

const withTargetValue = (action) => (state, payload) => {
  if (payload.target && payload.target.value)
    return [action, payload.target.value]
  return state
}

export const textInput = (props) =>
  input({
    ...props,
    type: "text",
    value: props.value,
    oninput: withTargetValue(props.oninput),
    onkeypress: withEnterKey(props.ondone)
  })

export const editable = ({ editing, ...inputProps }, content) =>
  editing ? textInput({ ...inputProps, onblur: inputProps.ondone }) : content

export const list = ({ items, render }) =>
  ul(items.map((value, index) => li(render(value, index))))
